/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: Session.java,v 1.11 2005/09/12 12:05:17 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej;

import java.lang.reflect.Constructor;
import java.util.Map;
import java.util.Properties;
import java.util.WeakHashMap;

import org.fossilec.odettej.service.Service;
import org.fossilec.odettej.service.TransferMode;
import org.fossilec.odettej.transport.ClassloaderTransportFactory;
import org.fossilec.odettej.transport.Transport;
import org.fossilec.odettej.transport.TransportFactory;

/**
 * The Session class represents a session between Odette FTP entities and is not
 * subclassed.
 * 
 * @author Rafael Marins
 */
public final class Session {

	/**
	 * Set of constants used by this API.
	 */
	private static final Properties LOCAL_CONSTANTS = new Properties();

	static {
		LOCAL_CONSTANTS.put("odette.capabilities.compression", "true");
		LOCAL_CONSTANTS.put("odette.capabilities.restart", "true");
		LOCAL_CONSTANTS.put("odette.capabilities.initial-state", "Both");
		LOCAL_CONSTANTS.put("odette.capabilities.mode", TransferMode.BOTH
				.toString());

		/* The maximum exchange buffer size allowed by this implementation */
		LOCAL_CONSTANTS.put("odette.capabilities.max-buffer", "99999");
		LOCAL_CONSTANTS.put("odette.capabilities.max-window", "999");
	};

	/**
	 * Being used to store local variables and api information
	 */
	private Properties properties;

	private AbstractState state;

	private VirtualFile file;

	private Service service;

	private PeerAuthenticator authenticator = null;

	private ListenerTemplate listenerSupport = null;

	private Map stateClassMap = new WeakHashMap(1, 2);

	/**
	 * Class constructor with a given property list used for the new Session.
	 * 
	 * @param props
	 *            Properties object with all relevant properties.
	 */
	private Session(Properties props) {
		super();

		if (props == null) {
			throw new IllegalArgumentException("Properties must not be null.");
		}

		PreferencesUtil.fillDefaultVariables(props);

		this.properties = props;
	}

	/**
	 * Get a new Session object.<br>
	 * 
	 * @param props
	 *            Properties object that holds relevant properties.
	 * @return A new Session object.
	 */
	public static Session getInstance(Properties props) {
		Session ss = new Session(props);
		return ss;
	}

	public static Session getInstance(Properties props, TransferMode mode) {
		Session ss = new Session(props);
		PreferencesUtil.setTransferMode(props, mode);

		return ss;
	}

	public static Session getInstance(Properties props, boolean restart) {
		Session ss = new Session(props);
		PreferencesUtil.setRestartSupport(props, restart);

		return ss;
	}

	public static Session getInstance(Properties props, TransferMode mode,
			boolean restart) {

		Session ss = new Session(props);
		PreferencesUtil.setTransferMode(props, mode);
		PreferencesUtil.setRestartSupport(props, restart);

		return ss;
	}

	public int getMaxBuffer() {
		String maxBuffer = LOCAL_CONSTANTS
				.getProperty("odette.capabilities.max-buffer");

		return Integer.parseInt(maxBuffer);
	}

	public int getMaxWindow() {
		String maxWindow = LOCAL_CONSTANTS
				.getProperty("odette.capabilities.max-window");

		return Integer.parseInt(maxWindow);
	}

	public boolean isCompressionCapable() {
		String compression = LOCAL_CONSTANTS
				.getProperty("odette.capabilities.compression");

		return Boolean.valueOf(compression).booleanValue();
	}

	public boolean isRestartCapable() {
		String restart = LOCAL_CONSTANTS
				.getProperty("odette.capabilities.restart");

		return Boolean.valueOf(restart).booleanValue();
	}

	public TransferMode getCapableTransferMode() throws OdetteFTPException {

		String cap = LOCAL_CONSTANTS.getProperty("odette.capabilities.mode");
		TransferMode mode = (TransferMode) TransferMode.parse(String
				.valueOf(cap.charAt(0)));

		return mode;
	}

	public void setReceivingSupport(ListenerTemplate provider) {
		this.listenerSupport = provider;
	}

	ListenerTemplate getListenerSupport() {
		return listenerSupport;
	}

	public void setAuthenticator(PeerAuthenticator provider) {
		this.authenticator = provider;
	}

	PeerAuthenticator getAuthenticator() {
		return authenticator;
	}

	VirtualFile getFile() {
		return file;
	}

	void setFile(VirtualFile vf) {
		this.file = vf;
	}

	public Service getService() {
		return service;
	}

	/**
	 * Searches for the property with the specified key in local constants and
	 * session property list. If it is not found, the default property value is
	 * returned. This method return <code>null</code> if the property is not
	 * found.
	 * 
	 * @param key
	 *            Key name of the property.
	 * @return The value of this property by the specified key.
	 */
	public String getProperty(String key) {
		return properties.getProperty(key);
	}

	/**
	 * Searches for the property with the specified key in local constants and
	 * session property list. If it is not found, the default property value is
	 * returned. This method return the specified default value if the property
	 * is not found.
	 * 
	 * @param key
	 *            Key name of the property.
	 * @param defaultValue
	 *            Default value which is returned.
	 * @return The value of this property by the specified key.
	 */
	public String getProperty(String key, String defaultValue) {
		return properties.getProperty(key, defaultValue);
	}

	/**
	 * Set a property in Session and return the previous property value. Giving
	 * a <code>null</code> value remove the existing property if exists.<br>
	 * 
	 * If the specified key correspond to any local constant this method will
	 * set nothing and return <code>null</code>.
	 * 
	 * @param key
	 *            Key name of the property.
	 * @param value
	 *            The value attribute to the specified key.
	 * @return Previous value of the specified property, or <code>null</code>
	 *         if it did not have one.
	 */
	public Object setProperty(String key, String value) {
		return properties.setProperty(key, value);
	}

	public Properties getProperties() {
		return properties;
	}

	public OdetteFTPEntity acceptConnection(Transport transport)
			throws OdetteFTPException {

		/* incoming network connection */

		if ((state == null) || (state instanceof IdleState)) {

			boolean stb = PreferencesUtil.getBoolean(properties, "odette.stb");
			int sdeb = PreferencesUtil.getInt(properties, "odette.buffer-size");

			service = Service.getInstance(transport, sdeb, stb);

			setProperty("odette.called-address", transport.getLocalAddress());
			setProperty("odette.calling-address", transport.getRemoteAddress());

			/* start session phase */
			changeState(ListenerState.class);
			state.startSession();

		} else {
			NotInIdleStateError();
		}

		return state;
	}

	public OdetteFTPEntity connect(String address, String userCode,
			String password) throws OdetteFTPException {

		if (state == null) {
			/* make network connection */
			Transport transport = getTransportInstance();

			boolean stb = PreferencesUtil.getBoolean(properties, "odette.stb");
			int sdeb = PreferencesUtil.getInt(properties, "odette.buffer-size");

			service = Service.getInstance(transport, sdeb, stb);

			service.connect(address);

			setProperty("odette.called-address", address);
			setProperty("odette.calling-address", transport.getLocalAddress());

			/* start session phase */
			PreferencesUtil.setLocalUser(properties, userCode);
			PreferencesUtil.setLocalPassword(properties, password);

			changeState(SpeakerState.class);
			state.startSession();

		} else {
			NotInIdleStateError();
		}

		return state;
	}

	private void NotInIdleStateError() {
		throw new IllegalStateException("Not in Idle state");
	}

	private Transport getTransportInstance() {
		String className = PreferencesUtil.getTransport(properties);
		TransportFactory factory = new ClassloaderTransportFactory(className);
		Transport transport = factory.createTransport();

		return transport;
	}

	void disconnect() throws OdetteFTPException {
		service.disconnect();
		changeState(IdleState.class);
	}

	AbstractState state() {
		if (state == null) {
			changeState(IdleState.class);
		}

		return state;
	}

	void changeState(Class type) {

		if (stateClassMap.containsKey(type)) {
			state = (AbstractState) stateClassMap.get(type);
		} else {
			state = loadState(type);
			stateClassMap.put(type, state);
		}
	}

	private AbstractState loadState(Class type) {
		Constructor ctr;
		AbstractState loaded = null;

		try {
			ctr = type.getConstructor(new Class[] { Session.class });
			loaded = (AbstractState) ctr.newInstance(new Object[] { this });

		} catch (Exception e) {
			e.printStackTrace();
		}

		return loaded;
	}

}
