/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: AnswerReason.java,v 1.5 2005/09/12 19:17:27 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.util.Collection;
import java.util.Map;
import java.util.WeakHashMap;

/**
 * Enumeration that represents each reason code for avaiable Answer Reasons
 * while transfering a Virtual File.
 * <p>
 * Answer Codes
 * 
 * <pre>
 *        Value: 01 - Invalid filename.
 *               02 - Invalid destination.
 *               03 - Invalid origin.
 *               04 - Storage record format not supported.
 *               05 - Maximum record length not supported.
 *               06 - File size is too big.
 *               10 - Invalid record count.
 *               11 - Invalid byte count.
 *               12 - Access method failure.
 *               13 - Duplicate file.
 *               99 - Unspecified reason.
 * </pre>
 */
public final class AnswerReason {

	/**
	 * Invalid filename.
	 */
	public static final AnswerReason INVALID_FILENAME = new AnswerReason("01");

	/**
	 * Invalid destination.
	 */
	public static final AnswerReason INVALID_DESTINATION = new AnswerReason(
			"02");

	/**
	 * Invalid origin.
	 */
	public static final AnswerReason INVALID_ORIGIN = new AnswerReason("03");

	/**
	 * Storage record format not supported.
	 */
	public static final AnswerReason UNSUPPORTED_STORAGE_RECORD_FORMAT = new AnswerReason(
			"04");

	/**
	 * Maximum record length not supported.
	 */
	public static final AnswerReason UNSUPPORTED_MAXIMUM_RECORD_LENGTH = new AnswerReason(
			"05");

	/**
	 * File size is too big.
	 */
	public static final AnswerReason FILE_SIZE_EXCEED = new AnswerReason("06");

	/**
	 * Invalid record count.
	 */
	public static final AnswerReason INVALID_RECORD_COUNT = new AnswerReason(
			"10");

	/**
	 * Invalid byte count.
	 */
	public static final AnswerReason INVALID_BYTE_COUNT = new AnswerReason("11");

	/**
	 * Access method failure.
	 */
	public static final AnswerReason ACCESS_METHOD_FAILURE = new AnswerReason(
			"12");

	/**
	 * Duplicate file.
	 */
	public static final AnswerReason DUPLICATE_FILE = new AnswerReason("13");

	/**
	 * Unspecified reason.
	 */
	public static final AnswerReason UNSPECIFIED = new AnswerReason("99");

	/**
	 * Map used to store typed enums.
	 */
	private static final Map ENUM_MAP = new WeakHashMap(11);

	/*
	 * Add each typed enum to Map on first class initialization.
	 */
	static {
		addEnum(INVALID_FILENAME);
		addEnum(INVALID_DESTINATION);
		addEnum(INVALID_ORIGIN);
		addEnum(UNSUPPORTED_STORAGE_RECORD_FORMAT);
		addEnum(UNSUPPORTED_MAXIMUM_RECORD_LENGTH);
		addEnum(FILE_SIZE_EXCEED);
		addEnum(INVALID_RECORD_COUNT);
		addEnum(INVALID_BYTE_COUNT);
		addEnum(ACCESS_METHOD_FAILURE);
		addEnum(DUPLICATE_FILE);
		addEnum(UNSPECIFIED);
	}

	private String reasonCode;

	/**
	 * Enumeration constructor where reason code is specified to satisfy
	 * ReasonCode interface.
	 * 
	 * @param aReasonCode
	 */
	private AnswerReason(String aReasonCode) {
		super();
		this.reasonCode = aReasonCode;
	}

	/**
	 * Return the protocol representation of AnswerReason enum.
	 * 
	 * @return <code>String</code> corresponding protocol code.
	 */
	public String getCode() {
		return this.reasonCode;
	}

	/**
	 * Convenient method for parsing the proper TransferMode instance given a
	 * identifier character.
	 * 
	 * @param code
	 *            The transfer mode character being evaluated
	 * @return CommandIdentifier Instance that correspond to the parameter
	 * @throws CommandNotRecognisedException
	 *             Command not recognised
	 */
	public static AnswerReason parse(String code) {
		return (AnswerReason) EnumUtil.parseEnum(ENUM_MAP, code);
	}

	/**
	 * Return a collection of the avaiable typed enumerations.
	 * 
	 * @return
	 */
	public static Collection values() {
		return ENUM_MAP.values();
	}

	/**
	 * Helper method to add a typed enum to the static Map.
	 * 
	 * @param type
	 *            Typed enum.
	 */
	private static void addEnum(AnswerReason type) {
		String code = type.getCode();
		ENUM_MAP.put(code, type);
	}
}
