/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: CommandIdentifier.java,v 1.7 2005/08/15 16:43:36 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.util.Collection;
import java.util.Map;
import java.util.WeakHashMap;

import org.fossilec.odettej.OdetteFTPException;

/**
 * Odette FTP entities communicate by sending and receiving messages in Exchange
 * Buffers. Each Exchange Buffer correspond to a command which is defined by the
 * Command Identifier. The identifier is verified at the first octet of an
 * Exchange Buffer that define the format of the remaing buffer.
 * <p>
 * The CommandIdentifier class provide type safe instances to distinguish every
 * Exchange Buffer over all supported commands types.
 */
public final class CommandIdentifier {

	/**
	 * Start Session Ready Message
	 */
	public static final CommandIdentifier SSRM = new CommandIdentifier("I");

	/**
	 * Start Session
	 */
	public static final CommandIdentifier SSID = new CommandIdentifier("X");

	/**
	 * Start File
	 */
	public static final CommandIdentifier SFID = new CommandIdentifier("H");

	/**
	 * Start File Positive Answer
	 */
	public static final CommandIdentifier SFPA = new CommandIdentifier("2");

	/**
	 * Start File Negative Answer
	 */
	public static final CommandIdentifier SFNA = new CommandIdentifier("3");

	/**
	 * Data
	 */
	public static final CommandIdentifier DATA = new CommandIdentifier("D");

	/**
	 * Set Credit
	 */
	public static final CommandIdentifier CDT = new CommandIdentifier("C");

	/**
	 * End File
	 */
	public static final CommandIdentifier EFID = new CommandIdentifier("T");

	/**
	 * End File Positive Answer
	 */
	public static final CommandIdentifier EFPA = new CommandIdentifier("4");

	/**
	 * End File Negative Answer
	 */
	public static final CommandIdentifier EFNA = new CommandIdentifier("5");

	/**
	 * End Session
	 */
	public static final CommandIdentifier ESID = new CommandIdentifier("F");

	/**
	 * Change Direction
	 */
	public static final CommandIdentifier CD = new CommandIdentifier("R");

	/**
	 * End to End Response
	 */
	public static final CommandIdentifier EERP = new CommandIdentifier("E");

	/**
	 * Ready To Receive
	 */
	public static final CommandIdentifier RTR = new CommandIdentifier("P");

	/**
	 * Map used to store typed enums.
	 */
	private static final Map ENUM_MAP = new WeakHashMap(14);

	/*
	 * Add each typed enum to Map on first class initialization.
	 */
	static {
		addEnum(SSRM);
		addEnum(SSID);
		addEnum(SFID);
		addEnum(SFPA);
		addEnum(SFNA);
		addEnum(DATA);
		addEnum(CDT);
		addEnum(EFID);
		addEnum(EFPA);
		addEnum(EFNA);
		addEnum(ESID);
		addEnum(CD);
		addEnum(EERP);
		addEnum(RTR);
	}

	private String code;

	/**
	 * Private constructor to evaluate the respective command identifier
	 * character regarding to the protocol specification.
	 * 
	 * @param Identifier
	 *            one valid command identifier character
	 */
	private CommandIdentifier(String aCode) {
		super();
		this.code = aCode;
	}

	/**
	 * Return the protocol representation of CommandIdentifier enum.
	 * 
	 * @return <code>String</code> corresponding protocol code.
	 */
	public String getCode() {
		return this.code;
	}

	public String toString() {
		return getCode();
	}

	/**
	 * Convenient method for parsing the proper CommandIdentifier instance given
	 * a identifier character.
	 * 
	 * @param identifier
	 *            The command identifier being evaluated
	 * @return CommandIdentifier Instance that correspond to the parameter
	 * @throws CommandNotRecognisedException
	 *             Command not recognised
	 */
	public static CommandIdentifier parse(String code)
			throws OdetteFTPException {

		CommandIdentifier ci = null;

		try {
			ci = (CommandIdentifier) EnumUtil.parseEnum(ENUM_MAP, code);
		} catch (IllegalArgumentException e) {
			throw new CommandNotRecognisedException("Command not recognised: "
					+ code);
		}

		return ci;
	}

	/**
	 * Helper method to add a typed enum to the static Map.
	 * 
	 * @param type
	 *            Typed enum.
	 */
	private static void addEnum(CommandIdentifier type) {
		ENUM_MAP.put(type.getCode(), type);
	}

	/**
	 * Return a collection of the avaiable typed enumerations.
	 * 
	 * @return
	 */
	public static Collection values() {
		return ENUM_MAP.values();
	}
}
