/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: CompressionMapping.java,v 1.3 2005/08/15 16:43:36 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.nio.ByteBuffer;

import org.fossilec.odettej.OdetteFTPException;

/**
 * Package visible class providing protocol compression mechanism for the Buffer
 * Mapping Strategy. This implementation provide an algorithm for filling
 * buffers when compression option is set at Odette FTP session setup.
 * 
 * @author Rafael Marins
 * @see org.fossilec.odettej.service.BufferMappingStrategy
 */
class CompressionMapping extends BufferMappingStrategy {

	public CompressionMapping(DataExchangeBuffer deb) {
		super(deb);
	}

	public int fill(ByteBuffer record, RecordFormat recordFormat,
			boolean lastRecord) throws OdetteFTPException {

		int total = 0;

		ByteBuffer buffer = getBuffer();

		ByteBuffer subrecord = ByteBuffer.allocateDirect(MAX_SUBRECORD_LENGTH);

		while ((buffer.remaining() >= MIN_SUBRECORD_LENGTH)
				&& (record.hasRemaining())) {

			subrecord.clear();

			byte header = drain(subrecord, record, buffer.remaining());

			subrecord.flip();

			total += addSubrecord(buffer, header, subrecord);
		}

		return total;
	}

	private byte drain(ByteBuffer toSubrecord, ByteBuffer fromRecord,
			int bufferAvaiable) {

		byte header;

		boolean compressing = false;
		boolean endOfRecord = false;

		byte readOctet = 0;
		byte lastOctet = 0;

		int limit = bufferAvaiable - 1;

		int max;

		if (limit < fromRecord.remaining()) {
			max = (limit > MAX_SUBRECORD_LENGTH ? MAX_SUBRECORD_LENGTH : limit);
		} else {
			max = (fromRecord.remaining() > MAX_SUBRECORD_LENGTH ? MAX_SUBRECORD_LENGTH
					: fromRecord.remaining());
		}

		int entryPosition = fromRecord.position();

		for (byte i = 1; i <= max; i++) {
			readOctet = fromRecord.get();

			if ((readOctet == lastOctet) && (i > 1)) {

				if ((i > 2) && (!compressing)) {
					fromRecord.position(fromRecord.position() - 2);
					toSubrecord.position(toSubrecord.position() - 1);
					break;
				} else {
					compressing = true;
				}

			} else {

				if ((readOctet != lastOctet) && (compressing)) {
					fromRecord.position(fromRecord.position() - 1);
					break;
				} else {
					toSubrecord.put(readOctet);
				}

			}

			lastOctet = readOctet;
		}

		int endPosition = fromRecord.position();
		byte count = (byte) (endPosition - entryPosition);

		endOfRecord = (!fromRecord.hasRemaining());

		header = createSubrecordHeader(endOfRecord, compressing, count);
		return header;
	}

	private static int addSubrecord(ByteBuffer deb, byte subrecordHeader,
			ByteBuffer subrecord) {

		subrecord.rewind();

		deb.put(subrecordHeader);
		deb.put(subrecord);

		return (subrecord.limit() + 1);
	}
}
