/**
 *  OdetteJ - Open Source Odette FTP implementation in Java(TM).
 *  Copyright (C) 2005 Fossil E-Commerce, http://www.fossilec.com
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  ------------------------------------------------------------------------
 *  $Id: OdetteExchangeBuffer.java,v 1.9 2005/08/16 03:51:26 rmarins Exp $
 *  ------------------------------------------------------------------------
 */

package org.fossilec.odettej.service;

import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.nio.charset.IllegalCharsetNameException;

import org.fossilec.odettej.OdetteFTPException;

/**
 * The communication between Odette FTP entities is basically done by sending
 * and receiving messages in Exchange Buffers via the transport layer. An Odette
 * Exchange Buffer can have either a Command or Data buffer.
 * 
 * @author Rafael Marins
 */
public abstract class OdetteExchangeBuffer {

	/**
	 * Minimum Exchange Buffer length allowed by protocol specification.
	 */
	public static final int MIN_OEB_LENGTH = 128;

	/**
	 * Maximum Exchange Buffer length allowed by protocol specification.
	 */
	public static final int MAX_OEB_LENGTH = 99999;

	protected static final String BUFFER_CHARSET = "ISO646-US";

	/**
	 * The content of this Exchange Buffer. Its buffer capacity are allocated
	 * before being initialized with some content.
	 */
	protected ByteBuffer buffer;

	/**
	 * Exchange Buffer length. It's initialized with the specified buffer
	 * capacity by the class constructor.
	 */
	private int length;

	/**
	 * The first octet of an Exchange Buffer is the Command Identifier that
	 * defines the format of the buffer.
	 */
	private CommandIdentifier identifier;

	protected OdetteExchangeBuffer() {
	}

	/**
	 * Return the corresponding <code>CommandIdentifier</code> relative to the
	 * Exchange Buffer contents. Its value is parsed from the first octet of the
	 * Exchange Buffer specified through class constructor.
	 * <p>
	 * The CommandIdentifier defines the format of the Exchange Buffer contents.
	 * 
	 * @return CommandIdentifier Instance relative to the command type of this
	 *         Exchange Buffer.
	 * @throws CommandNotRecognisedException
	 *             Command identifier not recognised in the specified buffer.
	 * @throws IllegalCharsetNameException
	 *             Error while decoding byte buffer with a invalid charset.
	 * @see org.fossilec.odettej.service.CommandIdentifier
	 */
	public CommandIdentifier getIdentifier() throws OdetteFTPException {
		if (identifier == null) {
			identifier = getIdentifier(getBuffer());
		}
		return identifier;
	}

	protected void setIdentifier(CommandIdentifier identifier) {
		this.identifier = identifier;
		BufferUtil.putAtPosition(buffer, new byte[] { (byte) identifier
				.toString().charAt(0) }, 0);
	}

	/**
	 * Return the Exchange Buffer rewinded.
	 * 
	 * @return ByteBuffer
	 */
	public ByteBuffer getBuffer() {
		return (ByteBuffer) buffer.rewind();
	}

	protected void setBuffer(ByteBuffer buffer) {
		this.buffer = buffer;
	}

	/**
	 * 
	 * 
	 * @param rawBuffer
	 * @param bufferLength
	 * @throws IllegalArgumentException
	 *             If the buffer is null.
	 * @throws OdetteFTPException
	 * @throws BufferCorruptedException
	 *             When a invalid buffer length is specified or with a wrong
	 *             position.
	 */
	protected abstract void parse(ByteBuffer rawBuffer)
			throws OdetteFTPException;

	/**
	 * Get the Odette Exchange Buffer length.
	 * 
	 * @return The Odette Exchange Buffer length.
	 */
	public int getLength() {
		return length;
	}

	/**
	 * Set the Odette Exchange Buffer length.
	 * 
	 * @param length
	 *            The length of Odette Exchange Buffer.
	 */
	protected void setLength(int length) {
		this.length = length;
	}

	/**
	 * Create and return a new OdetteExchangeBuffer instance corresponding to a
	 * Command Exchange Buffer or Data Exchange Buffer, regarding the identifier
	 * at first octet of the exchange buffer data.
	 * 
	 * @param rawBuffer
	 *            The overall Exchange Buffer.
	 * @param bufferLength
	 *            Length of the remaining buffer to be read.
	 * @return ExchangeBuffer Concrete instance representing a Command Exchange
	 *         Buffer or Data Exchange Buffer.
	 * @throws OdetteFTPException
	 * @throws CommandNotRecognisedException
	 *             Command identifier not recognised.
	 * @throws BufferCorruptedException
	 *             When a invalid buffer length is specified or the buffer is
	 *             badly positioned.
	 * @see org.fossilec.odettej.service.CommandExchangeBuffer
	 * @see org.fossilec.odettej.service.DataExchangeBuffer
	 */
	public static OdetteExchangeBuffer getInstance(ByteBuffer rawBuffer,
			boolean strictFormat) throws OdetteFTPException {

		OdetteExchangeBuffer instance;

		byte firstOctet = rawBuffer.get();

		String code = wrapString(firstOctet, BUFFER_CHARSET);
		CommandIdentifier ci = CommandIdentifier.parse(code);

		// rewind one position
		rawBuffer.position(rawBuffer.position() - 1);

		if (ci == CommandIdentifier.DATA) {
			instance = new DataExchangeBuffer(rawBuffer);
		} else {
			instance = new CommandExchangeBuffer(rawBuffer, strictFormat);
		}

		return instance;
	}

	private static String wrapString(byte octet, String charset) {
		return wrapString(new byte[] { octet }, charset);
	}

	/**
	 * 
	 * @param octets
	 * @param charset
	 * @return
	 * @throws IllegalCharsetNameException
	 *             Error while decoding byte buffer with a invalid charset.
	 */
	private static String wrapString(byte[] octets, String charset) {
		String wrapped = null;

		try {
			wrapped = new String(octets, charset);

		} catch (UnsupportedEncodingException e) {
			throw new IllegalCharsetNameException("Invalid charset: " + charset);
		}

		return wrapped;

	}

	private static byte[] unwrapString(String text, String charset) {
		byte[] unwrapped;

		try {
			unwrapped = text.getBytes(charset);
		} catch (UnsupportedEncodingException e) {
			throw new IllegalCharsetNameException("Invalid charset: " + charset);
		}

		return unwrapped;
	}

	static byte[] unwrapString(String text) {
		return unwrapString(text, BUFFER_CHARSET);
	}

	static CommandIdentifier getIdentifier(ByteBuffer buffer)
			throws OdetteFTPException {
		String code = wrapString(buffer.get(), BUFFER_CHARSET);
		CommandIdentifier ci = CommandIdentifier.parse(code);
		return ci;
	}
}